﻿:Class  ADOC
⍝ User Command script for "ADOC"
⍝ Assumes that the "real" ADOC script is to be found where the registry key
⍝ 'HKEY_CURRENT_USER\Software\Dyalog\Dyalog APL/W 12.1 Unicode\SALT\CodeFolder'
⍝ is pointing to.
⍝ Kai Jaeger
⍝ Version 1.2 from 2011-08-04

    ∇ r←List;⎕IO;⎕ML ⍝ this function usually returns 1 or more namespaces (here only 1)
      :Access Shared Public
      ⎕IO←⎕ML←1
      r←⎕NS¨3⍴⊂''                                        ⍝ create ... commands
      r.Name←'ADOC.Browse' 'ADOC.List' 'ADOC.Help'       ⍝ their names
      r[1].Desc←'Creates full documentation and displayes it in the default browser'
      r[2].Desc←'Prints syntax of all public stuff to the session'
      r[3].Desc←'Displays help regarding ADOC'
      r.Group←⊂''                                   	 ⍝ 
     ⍝ Parsing rules for each:
      r[1].Parse←'1'        ⍝ -1- takes 1 argument
      r[2].Parse←'1'        ⍝ -2- takes 1 argument
      r[3].Parse←''         ⍝ -3- takes no rule
    ∇

    ∇ r←Run(Cmd Args);browser;cmdparser;cs;ref;⎕IO;⎕ML;path;regKey
      :Access Shared Public
      ⎕IO←⎕ML←1
      regKey←'HKEY_CURRENT_USER\Software\Dyalog\Dyalog APL/W 12.1 Unicode\SALT\CodeFolder'
      path←ReadRegKey regKey
      path,←'\adoc.dyalog'
      ⎕FIX ReadUtf8File path
     
      :Select Cmd
      :Case 'ADOC.Browse'
          ref←1⊃Args.Arguments
          :If '#'≠1⍴⍕ref
              ref←'#.',⍕ref
          :EndIf
          :If 3≠⎕NC ref
              ref←⍎ref
          :EndIf
          ADOC.Browse ref
          r←'Watch your browser'
      :Case 'ADOC.List'
          ref←1⊃Args.Arguments
          :If '#'≠1⍴⍕ref
              ref←'#.',⍕ref
          :EndIf
          r←'Full'ADOC.List(⍎ref)
      :Case 'ADOC.Help'
          r←ADOC.Browse ADOC
      :EndSelect
    ∇

    ∇ r←Help Cmd;⎕IO;⎕ML;sep
      ⎕IO←⎕ML←1
      sep←⎕TC[3]
      :Access Shared Public
      :Select Cmd
      :Case 'ADOC.Browse'
          r←'Gathers all sort of information from a class script,',sep
          r,←'compiles an HTML page with these pieces of information',sep
          r,←'and displays it in a browser. Use the -b flag to specify',sep
          r,←'a browser different from your default browser.'
      :Case 'ADOC.List'
          r←'Prints a list with all sorts of information to the session',sep
          r,←'about all public stuff given in the script specified in the right argument'
      :Case 'ADOC.Help'
          r←'Same as ADOC.Browse ADOC: displays ADOC#''s internal help information',sep
      :EndSelect
    ∇

    ∇ r←ReadUtf8File filename;fno;noOfBytes;bytes;⎕IO;⎕ML
      ⎕IO←⎕ML←1
      r←''
      :Trap 22
          fno←filename ⎕NTIE 0
      :Else
          ('Could not read file: ',filename)⎕SIGNAL ⎕EN
      :EndTrap
      noOfBytes←⎕NSIZE fno
      bytes←⎕NREAD fno 83,noOfBytes
      ⎕NUNTIE fno
      ⍝ Make sure it is unsigned, and drop the UTF-8 marker bytes if present
      bytes+←256×bytes<0
      bytes↓⍨←3×239 187 191≡3↑bytes ⍝ drop a potential UTF-8 marker
      r←'UTF-8'⎕UCS bytes
      :If ∨/(⎕UCS 13 10)⍷r
          r←Split r
      :ElseIf ∨/r=⎕UCS 10
          r←(⎕UCS 10)Split r
      :EndIf
    ∇

      Split←{
          ⎕ML←⎕IO←1
          ⍺←⎕UCS 13 10 ⍝ Default is CR+LF
          (⍴,⍺)↓¨⍺{⍵⊂⍨⍺⍷⍵}⍺,⍵
      }

    ∇ r←{default}ReadRegKey key;wsh;⎕WX
                 ⍝ Read a registry key. Uses a particular default path which can be overridden _
                 ⍝ via the left argument
      :Access public shared
      ⎕WX←1
      default←{0<⎕NC ⍵:⍎⍵ ⋄ ''}'default'
      'wsh'⎕WC'OLEClient' 'WScript.Shell'
      :Trap 11
          r←wsh.RegRead key
      :Else
          r←default
      :EndTrap
    ∇

:EndClass